//----------------------------------------------------------------------------
//
// C++ Objects for Allegro's gui
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// DEGUI gui Builder main program
//----------------------------------------------------------------------------

#include <stdlib.h>
#include <stdio.h>
#include "allegro.h"
#include "degui.h"
#include "inifile.h"
#include "build_di.h"
#include "build_mu.h"

//----------------------------------------------------------------------------
int main (void)
   {
   // Initialise Allegro
   allegro_init();
   install_keyboard(); 
   install_mouse();
   install_timer();

   // Get stuff from the ini file
   INIfile ini("builder.ini");
   driver = ini("Graphics","driver",GFX_AUTODETECT);
   xres = ini("Graphics","xres",640);
   yres = ini("Graphics","yres",480);

   // Set up the initial graphics mode
   if(set_gfx_mode(driver,xres,yres,0,0)==-1)
      {
      // Put up defaut mode if set didn't work
      if(set_gfx_mode(old_driver,old_xres,old_yres,0,0)==-1)
         {
         // Aaagh, nothing works, try the basic VGA as a last resort
         if(set_gfx_mode(GFX_VGA,320,200,0,0)==-1)
            {
            // Could not set ANY video mode
            set_gfx_mode(GFX_TEXT,0,0,0,0);
            cerr << "Could not set video mode (even tried 13h)\n";
            exit(1);
            }
         }
      }
   set_palette(desktop_palette);

   // Declare the gui builder object
   gui_builder = new builder_dialog;
   if(gui_builder==NULL) degui_no_memory();

   // Setup the modification keys
   gui_builder->set_move_key(KEY_F1);
   gui_builder->set_resize_key(KEY_F2);
   gui_builder->set_pushback_key(KEY_F3);
   gui_builder->set_pullfront_key(KEY_F4);
   gui_builder->set_delete_key(KEY_F10);
   gui_builder->set_granularity(5);
   gui_builder->set_fg_color_key(KEY_F5);
   gui_builder->set_bg_color_key(KEY_F6);

   // Start the gui builder
   gui_builder->execute();

   // Clean up and go home
   delete gui_builder;

   ini.set("Graphics","driver",driver);
   ini.set("Graphics","xres",xres);
   ini.set("Graphics","yres",yres);
   ini.close();

	return 0;
   }
//----------------------------------------------------------------------------
