//----------------------------------------------------------------------------
#ifndef ALLEGRO_CPP_GUI_BUILDER_OBJECTS
#define ALLEGRO_CPP_GUI_BUILDER_OBJECTS
//----------------------------------------------------------------------------
#include "allegro.h"
#include "degui.h"
#include "descript.h"

//----------------------------------------------------------------------------
// Allegro dialog managing class for building
//----------------------------------------------------------------------------
class builder_dialog
         :private dialog
			{
			private:
               // Keys used to modify the dialog
               int move_key;
               int resize_key;
               int pushback_key;
               int pullfront_key;
               int delete_key;
               int fg_color_key;
               int bg_color_key;

               int granularity;

               // Setup the menu stuff
               menu file_menu;
               menu object_menu;
               menu options_menu;
               menu help_menu;
               menu allegro_menu;
               grid_object *grid;

               // Internal variables for changing the resolution
               int res_c, res_w, res_h;

               // Internal function to intercept some messages
               static int builder_dialog_proc (int, DIALOG*, int);

			protected:
					// Add an builder object to the dialog
					void add (dialog_object*);

         public:
					// Constructors and destructors
					builder_dialog (void);
					virtual ~builder_dialog (void);

 					// Do the dialog  - NOTE: no popup dialog for the builder dialog
					void execute (void);

               // Restart the dialog to accept new objects and/or changes
               void reset (void) { dialog::reset(); };

               // Change the dialog
               void redraw (const bool total = false) { dialog::redraw(total); };
               void close (void) { dialog::close(); };
               void resolution (const int c, const int w, const int h) { res_c = c; res_w = w; res_h = h; reset(); }
               void center (const int a = 6, const int b = MAXINT) { dialog::center(a,b); };
               void use_palette (PALETTE p) { dialog::use_palette(p); };

               // Object adding routines
               int add_box_object (void);
               int add_shadow_box_object (void);
               int add_bitmap_object (void);
               int add_text_object (void);
               int add_centerd_text_object (void);
               int add_edittext_object (void);
               int add_button_object (void);
               int add_checkbox_object (void);
               int add_radio_button_object (void);
               int add_list_object (void);
               int add_textbox_object (void);
               int add_user_object (void);

   				// Modify how the dialog works
               void set_move_key (const int);
               void set_resize_key (const int);
               void set_pushback_key (const int);
               void set_pullfront_key (const int);
               void set_delete_key (const int);
               void set_granularity (const int);
               void set_fg_color_key (const int);
               void set_bg_color_key (const int);

               int get_move_key (void) const { return move_key; };
               int get_resize_key (void) const { return resize_key; };
               int get_pushback_key (void) const { return pushback_key; };
               int get_pullfront_key (void) const { return pullfront_key; };
               int get_delete_key (void) const { return delete_key; };
               int get_granularity (void) const { return granularity; };
               int get_fg_color_key (void) const { return fg_color_key; };
               int get_bg_color_key (void) const { return bg_color_key; };

    				// Save and restore the positions of the objects in the dialog
               bool load (const char*, const bool = false);
               void save (const char*);

					// Export the dialog
					void export (const char*);
			};

//----------------------------------------------------------------------------
// The main dialog
extern builder_dialog *gui_builder;

//----------------------------------------------------------------------------
// Some helper functions
void adjust_object_color (int *color, const char *title);

//----------------------------------------------------------------------------
// Graphics settings
extern int driver;
extern int xres;
extern int yres;

extern int old_driver;
extern int old_xres;
extern int old_yres;

//----------------------------------------------------------------------------
#endif

