//----------------------------------------------------------------------------
//
// C++ Objects for Allegro's gui
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Functions for the main menu
//----------------------------------------------------------------------------
#include "build_mu.h"
#include "build_di.h"
#include "descript.h"
#include <stdio.h>
#include <string.h>

//-----------------------------------------------------------------------------
// Export to a dialog
int dialog_export (void)
   {
   // Get an export filename
   char filename[80];
   strcpy(filename,"");
   int ret = file_select("Export C++ objects to file",filename,"cc");

   // If it wasn't cancelled then export the thing
   if(ret!=0) gui_builder->export(filename);
   gui_builder->redraw();
   return D_O_K;
   }
//----------------------------------------------------------------------------
// Save and load a partially done dialog
int save_dialog (void)
   {
   // Get an save filename
   char filename[80];
   strcpy(filename,"");
   int ret = file_select("Save to file",filename,"adf");

   // If it wasn't cancelled then save the thing
   if(ret!=0)
      {
      delete_file(filename);
      gui_builder->save(filename);
      }
   gui_builder->redraw();
   return D_O_K;
   }
//----------------------------------------------------------------------------
int load_dialog (void)
   {
   // Get an load filename
   char filename[80];
   strcpy(filename,"");
   int ret = file_select("Load from file",filename,"adf");

   // If it wasn't cancelled then load the thing
   if(ret!=0)
      {
      // Try to load the file
      if(gui_builder->load(filename)==false)
         {
         // Tell the user if it didn't work
         alert(NULL,"Problem loading file",NULL,"&Ok",NULL,'o',0);
         }
      }
   gui_builder->reset();
   return D_O_K;
   }
//----------------------------------------------------------------------------
int append_dialog (void)
   {
   // Get an load filename
   char filename[80];
   strcpy(filename,"");
   int ret = file_select("Append from file",filename,"adf");

   // If it wasn't cancelled then load the thing
   if(ret!=0)
      {
      // Try to load the file
      if(gui_builder->load(filename,true)==false)
         {
         // Tell the user if it didn't work
         alert(NULL,"Problem appending file",NULL,"&Ok",NULL,'o',0);
         }
      }
   return D_O_K;
   }

//----------------------------------------------------------------------------
// Close the dialog from the menu
int main_dialog_close (void)
   {
   // Ask if they want to save at the exit
   if(alert(NULL,"Save dialog before exit?",NULL,"&Yes","&No",'n','n')==1)
      {
      save_dialog();
      }

   // As the main dialog to close itself
   gui_builder->close();
   return D_O_K;
   }

//----------------------------------------------------------------------------
// General dialog procedures
//----------------------------------------------------------------------------
int set_fg_color (void)
   {
   adjust_object_color(&gui_fg_color,"Modify GUI foreground color");
   gui_builder->redraw();
   return D_O_K;
   }
//----------------------------------------------------------------------------
int set_mg_color (void)
   {
   adjust_object_color(&gui_mg_color,"Modify GUI midground color");
   gui_builder->redraw();
   return D_O_K;
   }
//----------------------------------------------------------------------------
int set_bg_color (void)
   {
   adjust_object_color(&gui_bg_color,"Modify GUI background color");
   gui_builder->redraw();
   return D_O_K;
   }
//----------------------------------------------------------------------------
int set_builder_palette (void)
   {
   char filename[80];
   strcpy(filename,"");
   if(file_select("Choose file to load palette from",filename,"bmp;tga;lbm;pcx")!=0)
      {
      // Find the type of file it is and load it
      BITMAP *b;
      PALETTE pal;
      if(stricmp(get_extension((char*)filename),"bmp")==0) b = load_bmp((char*)filename,pal);
      else if(stricmp(get_extension((char*)filename),"tga")==0) b = load_tga((char*)filename,pal);
      else if(stricmp(get_extension((char*)filename),"lbm")==0) b = load_lbm((char*)filename,pal);
      else b = load_pcx((char*)filename,pal);

      // Adjust the gui for the best colors
      gui_builder->use_palette(pal);

      destroy_bitmap(b);
      }
   return D_O_K;
   }
//----------------------------------------------------------------------------
int center_dialog (void)
   {
   gui_builder->center();
   gui_builder->redraw();
   return D_O_K;
   }
//----------------------------------------------------------------------------
int placement_granularity (void)
   {
   char gran_string[10];
   sprintf(gran_string,"%i",gui_builder->get_granularity());

   // Object definitions
   shadow_box_object object_0;
   button_object object_1("&Ok",0,true);
   button_object object_2("&Cancel",0,true);
   text_object title("Set placemet grid");
   text_object grid_size("Grid size:");
   edittext_object gran_edit(gran_string,4);

   // Add Objects to the dialog
   dialog the_dialog;
   the_dialog.add(object_0,165,160,305,160);
   the_dialog.add(object_1,215,285,80,20);
   the_dialog.add(object_2,350,285,80,20);
   the_dialog.add(title,244,170,136,8);
   the_dialog.add(grid_size,192,224,80,8);
   the_dialog.add(gran_edit,273,224,40,8,gui_fg_color,gui_bg_color);
   the_dialog.center();

   // Ask the user to change the granularity
   if(the_dialog.popup()==&object_1)
      {
      // Change the granularity according to the user
      int gran = atoi(gran_string);
      if(gran<1) gran = 1;

      gui_builder->set_granularity(gran);
      }
   return D_O_K;
   }
//----------------------------------------------------------------------------
int choose_resolution (void)
   {
   int res_c, res_w, res_h;
   if(gfx_mode_select(&res_c,&res_w,&res_h)!=0)
      {
      gui_builder->resolution(res_c,res_w,res_h);
      }
   else
      {
      gui_builder->redraw();
      }
   return D_O_K;
   }

//----------------------------------------------------------------------------
// Help procedures
//----------------------------------------------------------------------------
int help_procedure (void)
   {
   // Uhh... it's a bit difficult to read like this....
   char *helptext =
"
            KEY DEFINITIONS

Here are the keys that may be useful during the building of your dialog.  Put the mouse over the object you want to change and then press one of the following keys.  Note that you DON'T have to click the mouse.

<F1>	Move object
Click to put the object back down

<F2>	Resize object
Click when the object is the desired size

Caution: Don't resize a textbox_object or listbox to be really narrow or the builder will crash.  I don't know what happens but you have the source, so you tell me.  It's not useful to make super narrow textbox_objects or list_object so you won't make one, right?

<F3>	Push object back
This pushes the object lower in the dialog and it will be redraw earlier whenever the dialog is redrawn.  It sort of pushed the object behind other objects.

<F4>  Pull object to front
This pull the object higher in the dialog and it will be redraw later whenever the dialog is redrawn.  It sort of pulled the object infront of other objects.

<F5>  Change an object foreground color
You can specify the color in RGB values or palette indices.

<F6>  Change an object background color
You can specify the color in RGB values or palette indices.

<F10>  Delete an object
This removes the object from the current dialog



            MENU DEFINITIONS


            File menu
[Load]
Load a DEGUI builder file (ADF)

[Append]
Append an ADF file to the current objects

[Save]
Save the current objects to an ADF file

[Export C++]
Export the current object to C++ code

[Exit]
Quit the DEGUI builder


            Object menu
[Box]
Create a new box object

[Shadow Box]
Create a new shadow box object

[Bitmap]
Create a new bitmap object

[Text]
Create a new text object

[Centerd text]
Create a new centerd text object

[Editable text]
Create a new editable text object

[Button]
Create a new button object

[Checkbox]
Create a new checkbox object

[Radio button]
Create a new radio button object

[List Box]
Create a new listbox object

[Textbox]
Create a new textbox object

[User]
Create a new user defined object


            Option menu

[Foreground color]
Change the default foreground color

[Midground color]
Change the default midground color

[Background color]
Change the default background color

[Palette]
Load a palette from a file and use it

[Center]
Center all the objects to the center of the screen

[Granularity]
Change the movement grid spacing

[Resolution]
Change the graphics resolution


            Help menu

[Help]
Simple explanation of how to use the program

[About]
Information about the program
";

   // Object definitions - DEGUI 1.0 beta
   shadow_box_object object_0;
   text_object object_1("Help - Escape to quit");
   textbox_object object_2(helptext,true,true);

   // Add Objects to the dialog
   dialog the_dialog;
   the_dialog.add(object_0,100,100,385,250);
   the_dialog.add(object_1,200,105,32,8);
   the_dialog.add(object_2,105,115,375,230);

   // Execute the help stuff
   the_dialog.center();
   the_dialog.popup();

   return D_O_K;
   }
//----------------------------------------------------------------------------
int about_procedure (void)
   {
   // Save the palette so we can replace it later
   PALETTE pal;
   get_palette(pal);

   char *filename = "about.pcx";

   char version[50];
   sprintf(version,"DEGUI %s",DEGUI_VERSION_STR);

   // Object definitions
   shadow_box_object object_0;
   bitmap_object picture(filename);
   text_object object_2("Doug Eleveld's");
   text_object object_3(version);
   text_object object_4("for Allegro");
   button_object object_5("Ok",13,true);
   text_object object_6("C++ GUI builder");

   // Setup the correct palette for the about bitmap
   PALETTE about;
   BITMAP *bit = load_pcx(filename,about);
   gui_builder->use_palette(about);
   gui_builder->redraw(true);

   // Add Objects to the dialog
   dialog the_dialog;
   the_dialog.add(object_0,234,129,171,264);
   the_dialog.add(picture,241,223,157,118);
   the_dialog.add(object_2,261,150,112,8);
   the_dialog.add(object_3,256,203,128,8);
   the_dialog.add(object_4,276,177,88,8);
   the_dialog.add(object_5,267,354,110,26);
   the_dialog.add(object_6,258,164,120,8);
   the_dialog.center();

   // Execute the dialog
   the_dialog.popup();
   destroy_bitmap(bit);

   // Put the palette back to what it was
   gui_builder->use_palette(pal);

   return D_O_K;
   }

//----------------------------------------------------------------------------
// Object adding procedures
//----------------------------------------------------------------------------
int add_box_object (void)
   {
   return gui_builder->add_box_object();
   }
//----------------------------------------------------------------------------
int add_shadow_box_object (void)
   {
   return gui_builder->add_shadow_box_object();
   }
//----------------------------------------------------------------------------
int add_bitmap_object (void)
   {
   return gui_builder->add_bitmap_object();
   }
//----------------------------------------------------------------------------
int add_text_object (void)
   {
   return gui_builder->add_text_object();
   }
//----------------------------------------------------------------------------
int add_centerd_text_object (void)
   {
   return gui_builder->add_centerd_text_object();
   }
//----------------------------------------------------------------------------
int add_edittext_object (void)
   {
   return gui_builder->add_edittext_object();
   }
//----------------------------------------------------------------------------
int add_button_object (void)
   {
   return gui_builder->add_button_object();
   }
//----------------------------------------------------------------------------
int add_checkbox_object (void)
   {
   return gui_builder->add_checkbox_object();
   }
//----------------------------------------------------------------------------
int add_radio_button_object (void)
   {
   return gui_builder->add_radio_button_object();
   }
//----------------------------------------------------------------------------
int add_list_object (void)
   {
   return gui_builder->add_list_object();
   }
//----------------------------------------------------------------------------
int add_textbox_object (void)
   {
   return gui_builder->add_textbox_object();
   }
//----------------------------------------------------------------------------
int add_user_object (void)
   {
   return gui_builder->add_user_object();
   }

//----------------------------------------------------------------------------

