//----------------------------------------------------------------------------
//
// C++ Objects for Allegro's gui
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Some C Objects for Allegro's gui
//----------------------------------------------------------------------------
#include "degui_c.h"
#include "allegro.h"

//----------------------------------------------------------------------------
// Find the object being pointed to
DIALOG* find_mouse_object (DIALOG* current)
   {
   DIALOG *pointed = NULL;
   int index;
   for(index=0;current[index].proc;index++)
      {
      // Find the object size and place
      int x = current[index].x;
      int y = current[index].y;
      int w = current[index].w;
      int h = current[index].h;
      int x2 = x+w;
      int y2 = y+h;

      // Get the mouse position
      int mx = mouse_x;
      int my = mouse_y;

      // Is the mouse pointing at the object
      if((mx>x)&&(my>y)&&(mx<x2)&&(my<y2))
         {
         // Remember the object
         pointed = &current[index];
         }
      }
   return pointed;
   }
//----------------------------------------------------------------------------
// Get the blank background from underneath an object
void get_object_background (BITMAP *bak, DIALOG *dia, DIALOG *pointed)
   {
   int i;
   show_mouse(NULL);
   for(i=0;dia[i].proc;i++)
      {
      if(&dia[i]!=pointed) dia[i].proc(MSG_DRAW,&dia[i],0);
      }

   // Get the background and draw the object
   blit(screen,bak,pointed->x,pointed->y,0,0,pointed->w+1,pointed->h+1);
   pointed->proc(MSG_DRAW,pointed,0);
   rect(screen,pointed->x,pointed->y,pointed->x+pointed->w,pointed->y+pointed->h,gui_fg_color);
   show_mouse(screen);
   }
//----------------------------------------------------------------------------
// DIALOG procedure for dialog to move objects
int object_move_proc (int msg, DIALOG *d, int c)
   {
   int gran;
   DIALOG *current;
   DIALOG *pointed = NULL;

   // Make sure that we are getting the right message
   if(msg!=MSG_XCHAR) return D_O_K;

   // Check for the keycode
   if ((c>>8) != d->key) return D_O_K;

   // Get the Allegro dialogs
   current = (DIALOG*)d->dp;
   if(current==NULL) return D_O_K;

   // Find the smallest object being pointed to
   pointed = find_mouse_object(current);

   // Setup the granularity
   gran = abs(d->d1);

   // Move the object where the user wants it
   return do_object_move(pointed,current,gran);
   }

//----------------------------------------------------------------------------
// DIALOG procedure for dialog to resize objects
int object_resize_proc (int msg, DIALOG *d, int c)
   {
   int gran;
   DIALOG *current;
   DIALOG *pointed = NULL;

   // Make sure that we are getting the right message
   if(msg!=MSG_XCHAR) return D_O_K;

   // Check for the keycode
   if ((c>>8) != d->key) return D_O_K;

   // Get the Allegro dialogs
   current = (DIALOG*)d->dp;
   if(current==NULL) return D_O_K;

   // Find the smallest object being pointed to
   pointed = find_mouse_object(current);

   // Setup the granularity
   gran = abs(d->d1);

   // Move the object where the user wants it
   return do_object_resize (pointed,current,gran);
   }
//----------------------------------------------------------------------------
// Helper function for dialog to move objects
int do_object_move (DIALOG *pointed, DIALOG *current, const int gran)
   {
   int original_x, original_y;
   int last_mousex, last_mousey, this_mousex, this_mousey;
   int done_move;
   BITMAP *bak;
   int ret = D_O_K;

   // Make sure that we pointed to an object
   if((pointed==NULL)||(current==NULL)) return D_O_K;

   // The original object position
   original_x = pointed->x;
   original_y = pointed->y;

   // Make sure that we pointed to an object
   if((pointed==NULL)||(current==NULL)) return D_O_K;
   position_mouse(pointed->x,pointed->y);

   // Move the mouse to the object
   last_mousex = (mouse_x/gran)*gran;
   last_mousey = (mouse_y/gran)*gran;
   this_mousex = (mouse_x/gran)*gran;
   this_mousey = (mouse_y/gran)*gran;

   // Get the object background
   bak = create_bitmap(pointed->w+1,pointed->h+1);
   get_object_background(bak,current,pointed);

   // Move the object where the user wants it
   done_move = FALSE;
   while(done_move==FALSE)
      {
      // Only redraw when the mouse has moved
      if((last_mousex!=this_mousex)||(last_mousey!=this_mousey))
         {
         // Replace with the background
         show_mouse(NULL);
         blit(bak,screen,0,0,pointed->x,pointed->y,pointed->w+1,pointed->h+1);

         // Move the object over
         pointed->x = this_mousex;
         pointed->y = this_mousey;

         // Get the new background
         blit(screen,bak,pointed->x,pointed->y,0,0,pointed->w+1,pointed->h+1);

         // Draw the object
         pointed->proc(MSG_DRAW,pointed,0);
         rect(screen,pointed->x,pointed->y,pointed->x+pointed->w,pointed->y+pointed->h,gui_fg_color);
         show_mouse(screen);

         // Remember where the mouse was
         last_mousex = this_mousex;
         last_mousey = this_mousey;
         }
      this_mousex = (mouse_x/gran)*gran;
      this_mousey = (mouse_y/gran)*gran;

      // Check for user keys
      if(keypressed())
         {
         switch(readkey()>>8)
            {
            // Check for user escape
            case(KEY_ESC):
               {
               // Replace the old position and return
               pointed->x = original_x;
               pointed->y = original_y;
               done_move = TRUE;
               break;
               }
            // Normal moving exit
            case(KEY_SPACE):
            case(KEY_ENTER):
               {
               done_move = TRUE;
               break;
               }
            // Check if it was the invoking key
            default:
               {
               };
            }
         }
      // Pressing the mouse will also set the object down
      else if(mouse_b)
         {
         done_move = TRUE;
         while(mouse_b) {};
         }
      }
   // Cleanup the keyboard buffer and the bitmaps
   destroy_bitmap(bak);

   // Redraw the object after moving the object
   ret = D_REDRAW | D_USED_CHAR;
   return ret;
   }

//----------------------------------------------------------------------------
// Helper function for dialog to resize objects
int do_object_resize (DIALOG *pointed, DIALOG *current, const int gran)
   {
   int original_w, original_h;
   int last_mousex, last_mousey, this_mousex, this_mousey;
   int done_resize;
   BITMAP *bak;
   int ret = D_O_K;

   // Make sure that we pointed to an object
   if((pointed==NULL)||(current==NULL)) return D_O_K;

   // The original object size
   original_w = pointed->w;
   original_h = pointed->h;

   // Move the mouse to the object
   position_mouse(pointed->x+pointed->w,pointed->y+pointed->h);
   last_mousex = (mouse_x/gran)*gran;
   last_mousey = (mouse_y/gran)*gran;
   this_mousex = (mouse_x/gran)*gran;
   this_mousey = (mouse_y/gran)*gran;
   set_mouse_range(pointed->x+1+gran,pointed->y+1+gran,SCREEN_W,SCREEN_H);

   // Get the object background
   bak = create_bitmap(pointed->w+1,pointed->h+1);
   get_object_background(bak,current,pointed);

   // Move the object where the user wants it
   done_resize = FALSE;
   while(done_resize==FALSE)
      {
      // Only redraw when the mouse has moved
      if((last_mousex!=this_mousex)||(last_mousey!=this_mousey))
         {
         // Replace with the background
         show_mouse(NULL);
         blit(bak,screen,0,0,pointed->x,pointed->y,pointed->w+1,pointed->h+1);

         // Resize the object
         pointed->w = this_mousex - pointed->x;
         pointed->h = this_mousey - pointed->y;

         // Resize the temporary bitmaps
         destroy_bitmap(bak);
         bak = create_bitmap(pointed->w+1,pointed->h+1);

         // Get the new background
         blit(screen,bak,pointed->x,pointed->y,0,0,pointed->w+1,pointed->h+1);

         // Draw the object
         pointed->proc(MSG_DRAW,pointed,0);
         rect(screen,pointed->x,pointed->y,pointed->x+pointed->w,pointed->y+pointed->h,gui_fg_color);
         show_mouse(screen);

         // Remember where the mouse was
         last_mousex = this_mousex;
         last_mousey = this_mousey;
         }
      this_mousex = (mouse_x/gran)*gran;
      this_mousey = (mouse_y/gran)*gran;

      // Check for user keys
      if(keypressed())
         {
         switch(readkey()>>8)
            {
            // Check for user escape
            case(KEY_ESC):
               {
               // Replace the old position and return
               pointed->w = original_w;
               pointed->h = original_h;
               done_resize = TRUE;
               break;
               }
            // Normal moving exit
            case(KEY_SPACE):
            case(KEY_ENTER):
               {
               done_resize = TRUE;
               break;
               }
            // Check if it was the invoking key
            default:
               {
               };
            }
         }
      // Pressing the mouse will also set the object down
      else if(mouse_b)
         {
         done_resize = TRUE;
         while(mouse_b) {};
         }
      }
   // Cleanup the keyboard buffer and the bitmaps
   destroy_bitmap(bak);

   set_mouse_range(0,0,SCREEN_W,SCREEN_H);

   // Redraw the object after moving the object
   ret = D_REDRAW | D_USED_CHAR;
   return ret;
   }

//----------------------------------------------------------------------------

