//----------------------------------------------------------------------------
//
// DEGUI - C++ Objects for Allegro's gui 1.1 beta
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------
#ifndef DEGUI_OBJECTS
#define DEGUI_OBJECTS

//----------------------------------------------------------------------------
#include <values.h>
#include "allegro.h"
//#include "tarray.h"

//----------------------------------------------------------------------------
// Version Info
#define DEGUI_VERSION          1
#define DEGUI_SUB_VERSION      1
#define DEGUI_VERSION_STR      "1.1 beta"
#define DEGUI_DATE_STR         "May 1997"

// Forward declaration for dialog_object parent type
class dialog;

//----------------------------------------------------------------------------
// This is the base object
//----------------------------------------------------------------------------
class dialog_object
			{
			private:
               // d pointer manipulation
               DIALOG *d;
               bool connected;
               bool in_c_dialog;

               // The parent dialog
               dialog *_parent;

               bool _redraw;

			protected:
               // Variables for the callback functions
               void(*all_callback)(void);
               int(*start_callback)(void);
               int(*end_callback)(void);
               int(*draw_callback)(void);
               int(*click_callback)(void);
               int(*dclick_callback)(void);
               int(*key_callback)(void);
               int(*char_callback)(int);
               int(*xchar_callback)(int);
               int(*wantfocus_callback)(void);
               int(*gotfocus_callback)(void);
               int(*lostfocus_callback)(void);
               int(*gotmouse_callback)(void);
               int(*lostmouse_callback)(void);
               int(*idle_callback)(void);
               int(*radio_callback)(int);
               int(*unknown_callback)(int);

               // Variables for handling the default behaviour for callbacks
               static int default_message;
               static dialog_object *default_object;
               static int default_data;
               static int default_answer;

   		public:
					// Constructors and destructors
					dialog_object (void);
					virtual ~dialog_object (void);

               // The main dispatching function i.e. the interface between the C and C++ object
					// This function calls the right member functions when used
					// an allegro gui procedure. You probably don't have to use it yourself
               static int object_message_proc (int, DIALOG*, int);

               // Message functions
               virtual void msg_start (void);
               virtual void msg_end (void);
               virtual void msg_draw (void);
               virtual void msg_click (void);
               virtual void msg_dclick (void);
               virtual void msg_key (void);
               virtual bool msg_char (const int);
               virtual bool msg_xchar (const int);
               virtual bool msg_wantfocus (void);
               virtual void msg_gotfocus (void);
               virtual void msg_lostfocus (void);
               virtual void msg_gotmouse (void);
               virtual void msg_lostmouse (void);
               virtual void msg_idle (void);
               virtual void msg_radio (const int);
               virtual void msg_unknown (const int);

               // Set the callback functions
               void set_all_callback       (void(*callback)(void)) { all_callback = callback; };
               void set_start_callback     (int(*callback)(void))  { start_callback = callback; };
               void set_end_callback       (int(*callback)(void))  { end_callback = callback; };
               void set_draw_callback      (int(*callback)(void))  { draw_callback = callback; };
               void set_click_callback     (int(*callback)(void))  { click_callback = callback; };
               void set_dclick_callback    (int(*callback)(void))  { dclick_callback = callback; };
               void set_key_callback       (int(*callback)(void))  { key_callback = callback; };
               void set_char_callback      (int(*callback)(int))   { char_callback = callback; };
               void set_xchar_callback     (int(*callback)(int))   { xchar_callback = callback; };
               void set_wantfocus_callback (int(*callback)(void))  { wantfocus_callback = callback; };
               void set_gotfocus_callback  (int(*callback)(void))  { gotfocus_callback = callback; };
               void set_lostfocus_callback (int(*callback)(void))  { lostfocus_callback = callback; };
               void set_gotmouse_callback  (int(*callback)(void))  { gotmouse_callback = callback; };
               void set_lostmouse_callback (int(*callback)(void))  { lostmouse_callback = callback; };
               void set_idle_callback      (int(*callback)(void))  { idle_callback = callback; };
               void set_radio_callback     (int(*callback)(int))   { radio_callback = callback; };
               void set_unknown_callback   (int(*callback)(int))   { unknown_callback = callback; };

               // The deafult behavious function for callbacks
               static void default_behaviour (void);

  					// Ask about the object state in a easy manner
               int x          (void) const { return d->x; };
               int y          (void) const { return d->y; };
               int w          (void) const { return d->w; };
               int h          (void) const { return d->h; };
               int fg         (void) const { return d->fg; };
               int bg         (void) const { return d->bg; };
               int key        (void) const { return d->key; };
               int flags      (void) const { return d->flags; };
					bool has_focus (void) const { if(d->flags&D_GOTFOCUS) return true; return false; }
					bool has_mouse (void) const { if(d->flags&D_GOTMOUSE) return true; return false; }
					bool hidden    (void) const { if(d->flags&D_HIDDEN  ) return true; return false; }
					bool will_exit (void) const { if(d->flags&D_EXIT    ) return true; return false; }
					bool selected  (void) const { if(d->flags&D_SELECTED) return true; return false; }
					bool disabled  (void) const { if(d->flags&D_DISABLED) return true; return false; }
               int d1         (void) const { return d->d1; };
               int d2         (void) const { return d->d2; };

               // Get the object parent dialog - used for redraws or such
               dialog* parent (void) const { return _parent; };

               // Change some stuff
               virtual void set_x     (const int i) { d->x = i; _redraw = true; };
               virtual void set_y     (const int i) { d->y = i; _redraw = true; };
               virtual void set_w     (const int i) { d->w = i; _redraw = true; };
               virtual void set_h     (const int i) { d->h = i; _redraw = true; };
               virtual void set_fg    (const int i) { d->fg = i; _redraw = true; };
               virtual void set_bg    (const int i) { d->bg = i; _redraw = true; };
               virtual void set_key   (const int i) { d->key = i; _redraw = true; };
               virtual void select    (void) { d->flags |= D_SELECTED; _redraw = true; };
               virtual void deselect  (void) { d->flags &= ~D_SELECTED; _redraw = true; };
               virtual void set_flag  (const int i) { d->flags |= i; _redraw = true;};
               virtual void clear_flag(const int i) { d->flags &= ~i; _redraw = true; };
               virtual void enable    (void) { d->flags &= ~D_DISABLED; _redraw = true; };
               virtual void disable   (void) { d->flags |= D_DISABLED; _redraw = true; };
               virtual void hide      (void) { d->flags |= D_HIDDEN; };
               virtual void unhide    (void) { d->flags &= ~D_HIDDEN; _redraw = true; };
               virtual void make_exit (void) { d->flags |= D_EXIT; _redraw = true; };
               virtual void dont_exit (void) { d->flags &= ~D_EXIT; _redraw = true; };
               virtual void set_d1    (const int i) { d->d1 = i; _redraw = true; };
               virtual void set_d2    (const int i) { d->d2 = i; _redraw = true; };

               virtual void redraw    (void) { _redraw = true; };

               virtual void place     (const int nx, const int ny) { set_x(nx); set_y(ny); };
               virtual void size      (const int nw, const int nh) { set_w(nw); set_h(nh); };
               virtual void shape     (const int nx, const int ny, const int nw, const int nh) { set_x(nx); set_y(ny); set_w(nw); set_h(nh); };

               // Create a DIALOG so that we can easily interface with C routines
               DIALOG create_DIALOG (const int = -1, const int = -1, const int = -1, const int = -1, const int = -1, const int = -1);

               // A pointer you can use for whatever you want
               void *dataptr;

               // DIALOG *d manipulation DANGEROUS !!!
               // Make sure you know what you are doing before you use these
               virtual void connect (DIALOG *);
               virtual void disconnect (void);

         // Let the proc object see the d pointer directly
         friend class proc_object;

         // Let the dialog adjust the parent
         friend class dialog;
			};

//----------------------------------------------------------------------------
// This is a wrapper class for classes that are based on allegro's stuff
//----------------------------------------------------------------------------
class proc_object
         :public dialog_object
			{
			private:
               // The related C procedure
               int(*dialog_procedure)(int,DIALOG*,int);

               // The address of the pointer that should be
               // passed as dp to the Allegro stuff
               // Keeping this as a pointer to a pointer allows derived
               // objects to modify the pointer and this class will
               // always point to the right place to get the pointer
               void **ptr_dp;

               // Helper function to pass messages to Allegro C functions
               // This is usefull for the C++ wrappers for Allegro C procedures
               // so that the C++ object just calls the C procedures
               int c_message (const int, const int = 0);

			protected:
               void setup (int(*pr)(int,DIALOG*,int), void **p = NULL) { dialog_procedure = pr; ptr_dp = p; };

   		public:
               // Message functions
               virtual void msg_start (void);
               virtual void msg_end (void);
               virtual void msg_draw (void);
               virtual void msg_click (void);
               virtual void msg_dclick (void);
               virtual void msg_key (void);
               virtual bool msg_char (const int);
               virtual bool msg_xchar (const int);
               virtual bool msg_wantfocus (void);
               virtual void msg_gotfocus (void);
               virtual void msg_lostfocus (void);
               virtual void msg_gotmouse (void);
               virtual void msg_lostmouse (void);
               virtual void msg_idle (void);
               virtual void msg_radio (const int);
               virtual void msg_unknown (const int);

					// Constructors and destructors
					proc_object (int(*pr)(int,DIALOG*,int), void **p = NULL);
					virtual ~proc_object (void);
			};

//----------------------------------------------------------------------------
// A simple object to clear the screen
//----------------------------------------------------------------------------
class clear_screen_object
			:public proc_object
			{
			private:

   		protected:

			public:
					// Constructors and destructors
					clear_screen_object (void);
					virtual ~clear_screen_object (void);
			};

//----------------------------------------------------------------------------
// Simple object just draws a box
//----------------------------------------------------------------------------
class box_object
			:public proc_object
			{
			private:

			protected:

			public:
					// Constructors and destructors
					box_object (void);
					virtual ~box_object (void);
			};

//----------------------------------------------------------------------------
// Simple object to draw a shadow box
//----------------------------------------------------------------------------
class shadow_box_object
			:public proc_object
			{
			private:

			protected:

			public:
					// Constructors and destructors
					shadow_box_object (void);
					virtual ~shadow_box_object (void);
			};

//----------------------------------------------------------------------------
// Simple object draws a bitmap
//----------------------------------------------------------------------------
class bitmap_object
			:public proc_object
			{
			private:

			protected:
 					// The bitmap to draw
            	BITMAP *b;
               bool allocated;

			public:
               // Message functions
               virtual void msg_draw (void);

					// Constructors and destructors
					bitmap_object (BITMAP* = NULL);
               bitmap_object (const char*);
					virtual ~bitmap_object (void);

               // Change the bitmap
               void set_bitmap (BITMAP *new_b) { b = new_b; redraw(); };
			};

//----------------------------------------------------------------------------
// Simple object draws some text
//----------------------------------------------------------------------------
class text_object
			:public proc_object
			{
			private:

			protected:
					// The text to print
					const char *text;

			public:
					// Constructors and destructors
					text_object (const char* = NULL);
					virtual ~text_object (void);

               // Modify the object text ont the fly
               void set_text (const char *t) { text = t; redraw(); };
			};

//----------------------------------------------------------------------------
// Simple object to draw some ceneterd text
//----------------------------------------------------------------------------
class centerd_text_object
			:public text_object
			{
			private:

			protected:

			public:
					// Constructors and destructors
					centerd_text_object (const char* = "Centerd text");
					virtual ~centerd_text_object (void);
			};

//----------------------------------------------------------------------------
// Button object
//----------------------------------------------------------------------------
class button_object
			:public text_object
			{
			private:

			protected:
					const bool willexit;
					const char shortcut;

			public:
					// Constructors and destructors
					button_object (const char* = "Button", const char = 0, const bool = false);
					virtual ~button_object (void);
			};

//----------------------------------------------------------------------------
// Simple check box object
//----------------------------------------------------------------------------
class checkbox_object
			:public button_object
			{
			private:

			protected:

			public:
					// Constructors and destructors
					checkbox_object (const char* = "Checkbox");
					virtual ~checkbox_object (void);
			};

//----------------------------------------------------------------------------
// Radio button object
//----------------------------------------------------------------------------
class radio_button_object
			:public button_object
			{
			private:

			protected:
               const int group;
               const bool square;

			public:
					// Constructors and destructors
					radio_button_object (const char* = "Radio button", const int = 0, const bool = false);
					virtual ~radio_button_object (void);
			};

//----------------------------------------------------------------------------
// Keyboard object
//----------------------------------------------------------------------------
class keyboard_object
			:public proc_object
			{
			private:
					int(*proc)(void);
               int keypress;

			protected:

			public:
					// Constructors and destructors
					keyboard_object (int(*pro)(void), const int key);
					virtual ~keyboard_object (void);
			};

//----------------------------------------------------------------------------
// Editable text string object
//----------------------------------------------------------------------------
class edittext_object
			:public proc_object
			{
			private:

			protected:
  					char *text;
               bool allocated;
					int max_length;

			public:
               // Message functions
               virtual void msg_draw (void);

					// Constructors and destructors
					edittext_object (char * = NULL, const int = 14);
					virtual ~edittext_object (void);

               // Set and get the text out
               void set_text (const char *);
               char* get_text (void) { return text; };
			};

//----------------------------------------------------------------------------
// Listbox object
//----------------------------------------------------------------------------
// Default list procedure with some simple numbers
extern char* default_list_object_procedure (int, int*);

class list_object
			:public proc_object
			{
			private:
					// The listing function
               char*(*list_grabber_proc)(int,int*);

			protected:

			public:
					// Constructors and destructors
					list_object (char*(*pro)(int,int*) = default_list_object_procedure);
					virtual ~list_object (void);

					// Getting some info out
					virtual int selection (void) { return d1(); }
			};

//----------------------------------------------------------------------------
// Textbox object
//----------------------------------------------------------------------------
// Default textbox text
extern char* default_textbox_object_text;

class textbox_object
         :public list_object
			{
			private:
               // The initial text and info
               char *text;
               int length;
               bool self_allocated;

               // Number of lines the the text
               int _lines;

               // Word wrapping variables
               bool wrapping;
               bool word_wrapping;

               // We have to monitor the width and the font
               // to make sure that we update the screen if
               // they are changed
               int width;
               int last_width;
               FONT* last_font;

               // Internal placement of line ends
               char **places;
               int allocated;

               // Allocation jumps for internal data expansion
               static int allocation_step;

               // Helper function for determining lines in the text
               char* find_line_end (char *string);

               // Buffering for the hidden characters
               char *buffered_place;
               char buffered_char;

               // Read line out of the text - Used by the list procedure
               char* get_line (const int);
               int lines (void) const { return _lines; };

               // The list procedure to get the text lines
               static textbox_object *current_textbox;
               static char* list_procedure (int, int*);

					// Protect anyone from seeing this inheretied function from the listbox
					virtual int selection (void) { return 0; };

			protected:

   		public:
               // Message functions
               virtual void msg_start (void);
               virtual void msg_end (void);
               virtual void msg_draw (void);
               virtual void msg_click (void);
               virtual void msg_dclick (void);
               virtual void msg_key (void);
               virtual bool msg_char (const int);
               virtual bool msg_xchar (const int);
               virtual bool msg_wantfocus (void);
               virtual void msg_gotfocus (void);
               virtual void msg_lostfocus (void);
               virtual void msg_gotmouse (void);
               virtual void msg_lostmouse (void);
               virtual void msg_idle (void);
               virtual void msg_radio (const int);
               virtual void msg_unknown (const int);

					// Constructors and destructors
					textbox_object (char*, const bool = false, const bool = false);
					virtual ~textbox_object (void);

               // Setup the text
               void set_text (char *tex);
			};

//----------------------------------------------------------------------------
// Wrapper for MENU struct
//----------------------------------------------------------------------------
class menu
			{
			private:
               MENU *internal;
               int number;

               void _add(const char*, int(*proc)(void), const MENU*);

			protected:

			public:
					// Constructors and destructors
					menu (void);
					virtual ~menu (void);

					// Add something to the menu
					void add (const char* = "", int(*proc)(void) = NULL);
               void add (const char*, const menu&);

               // The menu object needs to be able to acces the internal menu
               friend class menu_object;
			};

//----------------------------------------------------------------------------
// Menu object
//----------------------------------------------------------------------------
class menu_object
			:public proc_object
			{
			private:
               MENU *internal;

			protected:

			public:
               // Modification on menu object habits
               virtual void msg_gotmouse (void);

					// Constructors and destructors
					menu_object (const menu&);
					virtual ~menu_object (void);
			};

//----------------------------------------------------------------------------
// A simple object to clear the screen and put up a grid
//----------------------------------------------------------------------------
class grid_object
			:public dialog_object
			{
			private:
               int x_gran;
               int y_gran;

   		protected:

			public:
               // Message functions
               virtual void msg_draw (void);

					// Constructors and destructors
					grid_object (const int, const int);
					virtual ~grid_object (void);

               void set_granularity (const int x, const int y) { x_gran = x; y_gran = y; redraw(); };
			};

/*
//----------------------------------------------------------------------------
// An object to hold groups of objects NOTE: Undocumented !!!
//----------------------------------------------------------------------------
class dialog_object_group
			{
			private:

			protected:
               // The internal list of objects
               Tvector<dialog_object*> objects;

   		public:
					// Constructors and destructors
					dialog_object_group (void);
					dialog_object_group (dialog_object*, ...);
					virtual ~dialog_object_group (void);

               // Manipulation functions
               void add (dialog_object *obj) { objects.append(obj); };
//               void remove (dialog_object *obj) { objects.remove(obj); };
			};
*/

//----------------------------------------------------------------------------
// Allegro dialog managing class
//----------------------------------------------------------------------------
class dialog
			{
			private:

         protected:
               // The dialog array sent to Allegro
					DIALOG *object_dialogs;
               DIALOG *new_object_dialogs;
					int number;

               // Normal modification stuff
               bool restart;
               bool executing;
               bool _close;
               bool _redraw;

               // Scaling data
               int font_height;
               int font_width;

               // Private helper functions
               void find_extrema (const int, const int, int&, int&, int&, int&);

               // Internal function to intercept some messages
               static int dialog_proc (int, DIALOG*, int);

			public:
					// Constructors and destructors
					dialog (const int = 8, const int = 8);
					virtual ~dialog (void);

					// Add an object to the dialog
					void add (dialog_object&, const int = -1, const int = -1, const int = -1, const int = -1, const int = -1, const int = -1);
               void add (int(*proc)(int,DIALOG*,int), const int = 0,  const int = 0, const int = 0, const int = 0, const int = gui_fg_color, const int = gui_bg_color, const int = 0, const int = 0, const int = 0, const int = 0, void* = NULL );

               // Restart the dialog to accept new object_dialogs and/or changes
               void reset (void);

               // Change the dialog
               void redraw (const bool = false);
               void close (void);
               void use_palette (PALETTE);

					// Do the dialog
               void center (const int = 0, const int = MAXINT);
               void scale_to_font (FONT *);
					dialog_object* popup (const dialog_object* = NULL);
					dialog_object* execute (const dialog_object* = NULL);

               // Get some information about the dialogs
               // Used to interface with C DIALOG arrays for use with do_dialog()
               DIALOG *get_DIALOG (void) const { if(new_object_dialogs!=NULL) return new_object_dialogs; else return object_dialogs; };
			};

//----------------------------------------------------------------------------
// Some helper functions
//----------------------------------------------------------------------------
inline bool inside_rect(int x, int y,int x1,int y1,int x2,int y2)
   {
   if (x>=x1 && x<=x2 && y>=y1 && y<=y2) return true; else return false;
   }

//----------------------------------------------------------------------------
// Some global error handling procedures
//----------------------------------------------------------------------------
// General error handling procedure
void degui_error (const char *message);

// Simplified error handling
void degui_no_memory (void);

//----------------------------------------------------------------------------
#endif

