//----------------------------------------------------------------------------
//
// DEGUI - C++ Objects for Allegro's gui
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

/*
  Chained list class
  ----------------------------------------------------------
  (c) 1997 M.Baas
*/

#include "inifile.h"

/*----------------------------insert-----------------------------
  Insert node n in the list.
---------------------------------------------------------------*/
void list::insert(node *n)
{
  insert_last(n);
}


/*--------------------------insert_first-------------------------
  Insert node n as first node of the list.
---------------------------------------------------------------*/
void list::insert_first(node *n)
{
  n->succ=first;
  n->pred=NULL;
  if (first!=NULL) first->pred=n;
  first=n;
  if (last==NULL) last=n;
}

/*--------------------------insert_last--------------------------
  Insert node n as last node of the list.
---------------------------------------------------------------*/
void list::insert_last(node *n)
{
  n->succ=NULL;
  n->pred=last;
  if (last!=NULL) last->succ=n;
  last=n;
  if (first==NULL) first=n;
}

/*--------------------------insert_after-------------------------
  Insert node n after node p.

  Pre: p must be part of the list!
---------------------------------------------------------------*/
void list::insert_after(node *n, node *p)
{
  if (p==last) insert_last(n);
  else
  {
    n->succ = p->succ;
    n->pred = p;
    (n->succ)->pred=n;
    p->succ = n;
  };
}

/*--------------------------delete_node--------------------------
  Delete node n from the list without freeing n.

  Pre: n must be part of the list!
---------------------------------------------------------------*/
void list::delete_node(node *n)
{
  if (n==first)
  {
    first=n->succ;
    n->succ=NULL;
    // n->pred is already NULL since it's the first node
    if (first==NULL) last=NULL; else first->pred=NULL;
    return;
  };
  if (n==last)
  {
    last=n->pred;
    n->pred=NULL;
    // n->succ is already NULL since it's the last node
    if (last==NULL) first=NULL; else last->succ=NULL;
    return;
  };

  // Node is neither first nor last node.

  (n->succ)->pred = n->pred;
  (n->pred)->succ = n->succ;
  n->succ=NULL;
  n->pred=NULL;
}

/*--------------------------delete_list--------------------------
  Delete all nodes from the list without freeing them.
---------------------------------------------------------------*/
void list::delete_list()
{
  node *n = first;
  node *s;

  // Setting succ/pred from all nodes to NULL
  while(n!=NULL)
  {
    s=n->succ;
    n->succ=NULL;
    n->pred=NULL;
    n=s;
  };

  first=NULL;
  last=NULL;
};

/*----------------------------free_node--------------------------
  Delete node n from the list and free it.

  Pre: n must be part of the list!
---------------------------------------------------------------*/
void list::free_node(node *n)
{
  delete_node(n);
  delete n;
}

/*----------------------------free_list--------------------------
  Delete all nodes from the list and free them.
---------------------------------------------------------------*/
void list::free_list()
{
  node *n = first;
  node *s;

  // Free all nodes...
  while(n!=NULL)
  {
    s=n->succ;
    delete n;
    n=s;
  };

  first=NULL;
  last=NULL;
};

